#!/bin/bash -e
# This script tests the base image and invokes the secure baseline audit
# If any issues occur, it exits with non-zero exit code
VERBOSE=false
OSVER=$(grep Ubuntu /etc/issue | awk '{print $2}')
OSMAJORVER=$(echo $OSVER | cut -d "." -f 1)

function bail {
    echo "ERROR: $@ Build will now abort."
    exit 1
}

function verboseLog {
    if $VERBOSE; then
        echo "INFO: $@"
    fi
}

function bailIfFileMissing {
	file="$@"
	if [ -z "$file" ]; then
		bail "Unable to check if file is missing because no filename was specified [$file]."
	fi
	if [ ! -e "$file" ]; then
		bail "File [$file] does not exist."
	fi
    verboseLog "File [$file] exists"
}

while (( "$#" )); do
    if [ "$1" == "-v" ]; then
        VERBOSE=true
    fi
    shift
done

# BASE IMAGE TESTS
mdsd -V || bail "Error running mdsd -V. Build will now abort."
verboseLog "Ran mdsd -V"
service mdsd start || echo Service is already started - continuing
sleep 1
# Don't care if this generates an error, we care if logs are generated
bailIfFileMissing=/var/log/mdsd.err

service azsecd start || echo Service is already started - continuing
ps -C azsec-scan.py || bail "Azure security scanning service not started"
verboseLog "azsec-scan.py is running"
disabledScans=$(grep -L cycle=\"P /etc/azsec/cfg/*)
if [ -n "$disabledScans" ]; then
    bail "Scans are not enabled in scan file $disabledScans"
else
    verboseLog "azsecd scans are enabled"
fi

fqdn=$(hostname -f) || bail "Error running hostname -f. Build will now abort."

echo "FQDN = $fqdn"
if [[ $fqdn != *internal.cloudapp.net ]] && [[ $fqdn != *internal.chinacloudapp.cn ]]; then
    bail "Hostname -f [$fqdn] returns an FQDN ending with neither 'internal.cloudapp.net' nor 'internal.chinacloudapp.cn'"
else
    verboseLog "FQDN $fqdn matches the expected cloud suffixes"
fi

grep 'options timeout:1 attempts:5' /etc/resolv.conf || bail "Retry options not found in /etc/resolv.conf"
egrep '^hosts:\s+files\s+cache' /etc/nsswitch.conf || bail "unscd not enabled in /etc/nsswitch.conf"
egrep '^\s*enable-cache\s+hosts\s+yes' /etc/nscd.conf || bail "Host caching not enabled in /etc/nscd.conf"

source /etc/profile
if [ -z "$JAVA_HOME" ] || [ ! -d $JAVA_HOME ]; then
    bail "JAVA_HOME [$JAVA_HOME] is not a valid directory. Build will now abort"
else
    verboseLog "JAVA_HOME is a valid directory [$JAVA_HOME]"
fi

if [ $OSMAJORVER -eq 12 ]; then
    if ! which javac; then
        bail "Unable to locate javac (required on 12.04). Is the openjdk-7-jdk package installed?"
    else
        verboseLog "Ubuntu 12.04 - found javac: $(which javac)"
    fi
else
    verboseLog "Ubuntu 14.04 or newer - javac not required"
fi


ofilelimit=$(egrep -m 1 '^\*\s+hard\s+nofile' /etc/security/limits.conf | awk '{print $4}')
expectedfilelimit=10000
if [ -z "$ofilelimit" ]; then
    bail "No open file limit specified in /etc/security/limits.conf. Please specify a value of 10000 (nofile 10000)"
elif [ $ofilelimit -lt $expectedfilelimit ]; then
    bail "File limit [$ofilelimit] should be $expectedfilelimit"
else
    verboseLog "File limit [$ofilelimit] is >= $expectedfilelimit"
fi

pamLimitCount=$(egrep -c "^session\s+required\s+pam_limits.so" /etc/pam.d/common-session)
if [ $pamLimitCount -lt 1 ]; then
    bail "No lines found in /etc/pam.d/common-session that match 'session required pam_limits.so'"
else
    verboseLog "Session limits enabled in /etc/pam.d/common-session"
fi

# SECURE BASELINE TESTS
ovfEnvXml=/var/lib/waagent/ovf-env.xml
plaintextPass=$(egrep -c 'UserPassword>REDACTED|PublicKey' $ovfEnvXml)
if [ $plaintextPass -lt 1 ] ; then
    bail "Password was not redacted from $ovfEnvXml"
else
    verboseLog "Plaintext password not found in $ovfEnvXml"
fi

# Following test applies only to 12.04. Cleartext password bug is fixed in 14.04.
filename=$(find /var/lib/cloud/instances -name obj.pkl | head -n 1)
if [ $OSMAJORVER -eq 12 ]; then
    if [ -n "$filename" ]; then
        bail "$filename contains cleartext password and should be removed"
    else
        verboseLog "Ubuntu 12.04: obj.pkfl file not found in /var/lib/cloud/instances"
    fi
else
    verboseLog "Ubuntu 14.04+: Don't need to search for obj.pkl file in /var/lib/cloud/instances"
fi

# Confirm we comply with the baseline
results=$(azsec-scan.py -m -n bscan)
if ! echo $results | grep -q "0 errors and 0 findings"; then
    read -r -a failArray <<< $(echo $results | sed 's/|@|/\n/g' | grep FAIL | awk '{print $1}')
    # Loop through failures. If any of the failures doesn't match our permitted list, bail
    failCount=0
    for element in "${failArray[@]}";do
        if [[ "$element" =~ ^7= ]];then
            echo "OK to have some missing patches. Ignoring..."
        elif [ $OSMAJORVER -eq 12 ] && [[ "$element" =~ ^36= ]]; then
            echo "x11-common required for JDK in 12.04 image. Ignoring..."
        else
            # This finding isn't one we can ignore. Fail the build.
            echo $element
            let "failCount++"
        fi
    done
    if [ $failCount -gt 0 ]; then
        bail "Baseline security scan failed [$failCount failures]"
    fi
else
    verboseLog "Baseline scan completed successfully"
fi

# Confirm the artifacts for the unlock accounts script are present
unlockBaseName=unlock-accounts
bailIfFileMissing "/etc/cron.d/$unlockBaseName"
bailIfFileMissing "/usr/local/sbin/$unlockBaseName.sh"

# CLAMAV TEST
bailIfFileMissing "/var/lib/clamav/main.cvd"

echo "SUCCESS! All tests passed"
